#<profile4xi profile = "12" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "logger4xi.py"> 
#<abstract4xi abstract = "providing logger facilities to other parts of the package"/>
#<refer4xi reference = "util.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Logger4Xi', 'get_log_to_stderr4xi', 'set_log_to_stderr4xi', 'get_logger4xi','set_logger4xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import itertools
    import weakref
    import atexit         
except:
    pass
try:                 # rare ones
    pass          
except:
    pass 
try:                 # imported-as ones 
    pass           
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    pass
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi 
from constant_util4xi import ConstantUtil4Xi
from print4xi import print4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from time4xi import time4xi
#"
#/>


# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global _logger4xi
global _log_to_stderr4xi
#"
#/>


# -------------------------------------------------------------------------------------------------
# Definitions
# -------------------------------------------------------------------------------------------------
#<define4xi definition = "
_logger4xi = 'LOGGER4XI'
_log_to_stderr4xi = True
#"
#/>

# -------------------------------------------------------------------------------------------------
# Note                            
# ------------------------------------------------------------------------------------------------ 
#<note4xi note = "
# functions processing above global variables must locate inside this module !
# 
# 1) as separate functions
# 2) as class methods, which call above functions   "/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set-logger4xi" usage = "
#   sets global object _logger4xi
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def set_logger4xi(logger_param='LOGGER4XI'):                                                    
    '''
    sets global object _logger4xi
    '''
    global _logger4xi 

    class2print = "set_logger4xi"
    method2print = "set_logger4xi"
    
    message2print = "{started" + "; set-logger4xi-started: " + str(set_logger4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
       
    
    _logger4xi = logger_param

    message2print = "{set" + "; logger4xi-set-by-set-logger4xi to: " + str(_logger4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
    
    message2print = "{returning" + "; set-logger4xi-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
                  
#</function4xi>
  
# -------------------------------------------------------------------------------------------------
#<function4xi function = "get-logger4xi" usage = "
#
#   util.py
#       log_to_stderr(level=None):
#           global _log_to_stderr
#           import logging
#           logger = get_logger() <--- function call; class & object would be more natural because it gets an object 
#           formatter = logging.Formatter(DEFAULT_LOGGING_FORMAT)
#           handler = logging.StreamHandler()
#           handler.setFormatter(formatter)
#           logger.addHandler(handler) <--- object in use
#           if level is not None:
#               logger.setLevel(level) <--- object in use
#               _log_to_stderr = True
#
#   forking.py
#       prepare(data):
#           if 'log_to_stderr' in data and data['log_to_stderr']:
#               util.log_to_stderr()
#           if 'log_level' in data:
#               util.get_logger().setLevel(data['log_level']) <--- setLevel function
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def get_logger4xi():                                                    
    '''
    get_logger4xi.py function returns logger used by multiprocessing
    '''
    global _logger4xi

    class2print = "get_logger4xi"
    method2print = "get_logger4xi"
    
    message2print = "{started" + "; get-logger4xi-started: " + str(get_logger4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
   
  
    message2print = "{returning" + "; logger4xi-returning-by-get-logger4xi: " + str(_logger4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)   
        
         
        
    return _logger4xi
#</function4xi>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
#     if not _logger4xi:             
#                                              
        # XXX multiprocessing should cleanup before logging
#        if hasattr(atexit, 'unregister'):
#            atexit.unregister(_exit_function4xi)
#            atexit.register(_exit_function4xi)
#        else:
#            atexit._exithandlers.remove((_exit_function4xi, (), {}))
#            atexit._exithandlers.append((_exit_function4xi, (), {}))
#
#        _check_logger_class4xi()  
#        
#        cu2xi = ConstantUtil4Xi()
#        _logger4xi = logging.getLogger(cu2xi.get_logger_name()) # LOGGER_NAME_UTIL4XI dragonized
#
#    return _logger4xi
#"
#/>     

    
# -------------------------------------------------------------------------------------------------
# set_log_to_stderr4xi 
#<function4xi function = "set-log-to-stderr4xi" usage = "
#   sets global object log_to_stderr4xi
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def set_log_to_stderr4xi(log_to_stderr_param=True):                                                    
    '''
    sets global object log_to_stderr4xi
    '''
    global _log_to_stderr4xi
    
    class2print = "set_log_to_stderr4xi"
    method2print = "set_log_to_stderr4xi"
    
    message2print = "{started" + "; set-log-to-stderr4xi-started: " + str(set_log_to_stderr4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
   
  
        
    _log_to_stderr4xi = log_to_stderr_param   

    message2print = "{set" + "; log-to-stderr4xi-set-by-set-log-to-stderr4xi to: " + str(_log_to_stderr4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
    
    message2print = "{returning" + "; set-log-to-stderr4xi-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
                      
#</function4xi> 
        
# -------------------------------------------------------------------------------------------------
# get_log_to_stderr4xi 
#<function4xi function = "get-log-to-stderr4xi" usage = "
#   returns global object log_to_stderr4xi
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def get_log_to_stderr4xi():                                                    
    '''
    returns global object log_to_stderr4xi
    '''
    global _log_to_stderr4xi
    
    class2print = "get_log_to_stderr4xi"
    method2print = "get_log_to_stderr4xi"
    
    message2print = "{started" + "; get-log-to-stderr4xi-started: " + str(get_log_to_stderr4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
   
  
    message2print = "{returning" + "; log-to-stderr4xi-returning-by-get-log-to-stderr4xi: " + str(_log_to_stderr4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)   
        

    return _log_to_stderr4xi   
#</function4xi>         

 
# =====================================================================================================================
#<class4xi class = "Logger4Xi" base = "object" child = "None" synopsis = "offers methods for loggers" usage = "logging"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =====================================================================================================================
class Logger4Xi(object): #<-- inherits from object class
    '''
    offers methods for loggers 
    '''            
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('get_log_to_stderr', 'get_logger', 'set_logger',
                 'sense', 'reason', 'ace')          
    #"
    #/>
                                     
    
    # -------------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-Logger4Xi" usage = "initializes Logger4Xi">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # -------------------------------------------------------------------------------------------------   
    def __init__(self):
        '''
        initializes Logger4Xi
        '''
        class2print = 'Logger4Xi'
        method2print = '__init__'
        
        message2print = "{started" + "; init-of-Logger4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                         
        message2print = "{borning" + "; logger2xi-of-Logger4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{inheriting" + "; object-inheriting-by-logger2xi-of-Logger4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                                     
        try:                
            set_logger4xi(None)
        
            set_log_to_stderr4xi(False)
        
        except:
            message2print = "{hit" + "; init-hit-by-init-of-Logger4Xi: " + " *** " + str(__init__)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        
            pass
            
        message2print = "{activated" + "; logger2xi-of-Logger4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        message2print = "{inherited" + "; object-inherited-by-logger2xi-of-Logger4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                    
        message2print = "{returning" + "; init-of-Logger4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        

    #</methodize4xi>
        
        
    # -------------------------------------------------------------------------------------------------
    #<methodize4xi method = "set-logger-of-Logger4Xi" usage = "sets global object _logger4xi">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # -------------------------------------------------------------------------------------------------
    def set_logger(self, logger_param=None):                                                    
        '''
        set_logger method sets global object _logger4xi
        '''
        class2print = 'Logger4Xi'
        method2print = 'set_logger'
        
        
        message2print = "{started" + "; set-logger-of-Logger4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
       
        
        set_logger4xi(logger_param)
        
                    
        message2print = "{returning" + "; set-logger-of-Logger4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
     
    #</methodize4xi>
        
    # -------------------------------------------------------------------------------------------------
    #<methodize4xi method = "get-logger-of-Logger4Xi" usage = "logger used by multiprocessing">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # -------------------------------------------------------------------------------------------------
    def get_logger(self):                                                    
        '''
        get_logger method returns logger used by multiprocessing
        '''
        
        class2print = 'Logger4Xi'
        method2print = 'get_logger'
        
        
        message2print = "{started" + "; get-logger-of-Logger4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        message2print = "{returning" + "; get-logger4xi-function-returning-by-get-logger-of-Logger4Xi: " + str(get_logger4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                               
    
        return get_logger4xi()
    #</methodize4xi>
            
        
    # -------------------------------------------------------------------------------------------------
    #<methodize4xi method = "set-log-to-stderr-of-Logger4Xi" usage = "sets global object log_to_stderr4xi">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # -------------------------------------------------------------------------------------------------
    def set_log_to_stderr(self, log_to_stderr_param=False):                                                    
        '''
        sets global object log_to_stderr4xi
        '''
        
        class2print = 'Logger4Xi'
        method2print = 'set_log_to_stderr'
        
        
        message2print = "{started" + "; set-log-to-stderr-of-Logger4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
            
        set_log_to_stderr4xi(log_to_stderr_param)  
        
                    
        message2print = "{returning" + "; set-log-to-stderr-of-Logger4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
       
    #</methodize4xi>
               
    # -------------------------------------------------------------------------------------------------
    #<methodize4xi method = "get-log-to-stderr-of-Logger4Xi" usage = "returns log_to_stderr4xi">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # -------------------------------------------------------------------------------------------------
    def get_log_to_stderr(self):                                                    
        '''
        returns log_to_stderr4xi
        '''
        
        class2print = 'Logger4Xi'
        method2print = 'get_log_to_stderr'
        
        
        message2print = "{started" + "; get-log-to-stderr-of-Logger4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        message2print = "{returning" + "; get-log-to-stderr4xi-function-returning-by-get-log-to-stderr-of-Logger4Xi: " + str(get_log_to_stderr4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                             
              
        return get_log_to_stderr4xi()  
    #</methodize4xi>
                
                                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-Logger4Xi" usage = "senses Logger4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        senses Logger4Xi class objects
        '''
 
        
        class2print = 'Logger4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-Logger4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; Logger4Xi: " + str(Logger4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
    
        
        
      
        message2print = "{sensed" + "; Logger4Xi: " + str(Logger4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-Logger4Xi" usage = "reasons Logger4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        reasons Logger4Xi class objects
        '''
       
        
        class2print = 'Logger4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-Logger4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
      
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; Logger4Xi: "  + str(Logger4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{got" + "; kb4xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; Logger4Xi: "  + str(Logger4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)             
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-Logger4Xi" usage = "aces Logger4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        aces Logger4Xi class objects 
        '''
       
        
        
        class2print = 'Logger4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-Logger4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; Logger4Xi: " + "Logger4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; Logger4Xi: " + "Logger4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)             
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
 #</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of logger4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

    